﻿Public Class Main

#Region " Public Methods "
    Public Shared Sub Main(ByVal args() As String)
        If args Is Nothing OrElse args.Length = 0 Then
            'Show open file dialog
            Dim ofd As New OpenFileDialog
            Dim ret As DialogResult

            ofd.AddExtension = True
            ofd.CheckFileExists = True
            ofd.CheckPathExists = True
            ofd.DefaultExt = "8XV"
            ofd.Filter = "TI-83 Plus Files (*.8X*)|*.8X*|All Files (*.*)|*.*"
            ofd.FilterIndex = 0
            ofd.Multiselect = False
            ofd.Title = "Select TI-83 Plus File"
            ret = ofd.ShowDialog()

            If ret = DialogResult.OK Then
                _FixChecksum(ofd.FileName)

                _ShowSuccess()
            End If
        Else
            'Just fix the input file's checksum
            _FixChecksum(args(0))
        End If
    End Sub
#End Region

#Region " Private Methods "
    Private Shared Sub _FixChecksum(ByVal file As String)
        Dim fs As New IO.FileStream(file, IO.FileMode.Open, IO.FileAccess.ReadWrite)
        Dim s As New IO.BinaryWriter(fs)
        Dim bytes(2) As Byte
        Dim value As Integer
        Dim checksum As Integer

        'Seek to data size
        s.BaseStream.Seek(53, IO.SeekOrigin.Begin)
        s.BaseStream.Read(bytes, 0, 2)
        value = BitConverter.ToInt16(bytes, 0) And 65535

        'Loop through data bytes and calculate checksum
        For i As Integer = 0 To value - 1
            s.BaseStream.Read(bytes, 0, 1)
            bytes(1) = 0

            checksum = Convert.ToInt32(checksum) + _
                    Convert.ToInt16(BitConverter.ToInt16(bytes, 0) And 255)
            checksum = checksum And 65535
        Next

        'Seek to checksum area
        s.BaseStream.Seek(value + 55, IO.SeekOrigin.Begin)

        'Write the new checksum
        bytes = BitConverter.GetBytes(checksum)
        s.BaseStream.Write(bytes, 0, 2)

        s.Close()
    End Sub

    Private Shared Sub _ShowSuccess()
        MessageBox.Show("Checksum successfully written.", Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub
#End Region

End Class
